#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
from autoqa import rats, util
from autotest_lib.client.bin import utils
from autotest_lib.client.common_lib import error
from autotest_lib.client.bin.test_config import config_loader
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class rats_sanity(AutoQATest):
    version = 1

    @ExceptionCatcher("self.initialize_failed")
    def initialize(self, config, **kwargs):
        super(rats_sanity, self).initialize(config)
        self.irb_conf = os.path.join(self.bindir, 'irb.cfg')

    def setup(self):
        if os.path.exists(self.irb_conf):
            utils.system('yum -y install python-fedora')

    @ExceptionCatcher("self.run_once_failed")
    def run_once(self, baseurl, parents, name, **kwargs):
        if name:
            name = "%s-%s" % (name, util.get_basearch())
        else:
            name = baseurl
        os.chdir(self.bindir)
        cmd = "./sanity.py -s %s -l %s" % (self.tmpdir, self.resultsdir)
        cmd += " %s" % baseurl
        self.result = None
        try:
            out = utils.system_output(cmd + " 2>&1", retain_output=True)
        except error.CmdError, e:
            self.result = 'FAILED'
            out = e.result_obj.stdout

        if self.result is None:
            self.result = "PASSED"
        self.summary = "%s" % (name)
        self.highlights = None
        self.outputs = out

        # grab extra key = val data
        keyvals = util.read_keyvals()
        self.write_test_keyval(keyvals)
        if os.path.exists(self.irb_conf):
            irb_config = config_loader(self.irb_conf)
            testlist = ('repodata', 'comps',
                        'critpath_deps', 'critpath_integrity')
            try:
                rats.report_results(testlist, keyvals,
                                    url=irb_config.get('irb','url'),
                                    username=irb_config.get('irb','username'),
                                    password=irb_config.get('irb','password'))
            except Exception, e:
                print "Failed to send results to irb: %s" % str(e)

