#!/usr/bin/python

# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Vojtech Aschenbrenner <vaschenb@redhat.com>

import os
import sys
import operator
import autoqa.koji_utils
import autoqa.util
import rpmUtils.miscutils
from autoqa.repoinfo import repoinfo

import autoqa.util
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher
from autotest_lib.client.bin import utils
from autotest_lib.client.bin import test
from autotest_lib.client.bin.test_config import config_loader


class upgradepath(AutoQATest):
    version = 1 # increment if setup() changes
    test_result = 'PASS'
    log = []
    envr_list = set()

    def initialize(self, config):
        self.config = config_loader(config, self.tmpdir)
        #URL of logs/results stored on autotest-server
        self.autotest_url = autoqa.util.make_autotest_url(self.config)

    def setup(self):
        pass

    def compare(self, matching_build, tags, op, opstr):
        koji = autoqa.koji_utils.SimpleKojiClientSession()
        for tag in tags:
            latest_build = koji.latest_by_tag(tag, matching_build['name'])
            if latest_build is None:
                msg ='{0:<7}{1}'.format('[ OK ]', tag)
                self.log.append(msg)
                print msg

                msg = "\tNo such package: %s" % matching_build['name']
                self.log.append(msg)
                print msg
            else:
                result = autoqa.koji_utils.compareEVR(matching_build, latest_build)

                if op(result, 0):
                    ok = True
                    msg ='{0:<7}{1}'.format('[ OK ]', tag)
                    print msg
                    self.log.append(msg)
                else:
                    ok = False
                    self.test_result = 'FAIL'
                    msg ='{0:<7}{1}'.format('[FAIL]', tag)
                    print msg
                    self.log.append(msg)
                    self.envr_list.add(matching_build['envr'])
                msg = "\tLatest package: %s" % latest_build['nvr']
                self.log.append(msg)
                print msg
                if not ok:
                    msg = '\tFailed condition: Requested package %s Latest package' % opstr
                    self.log.append(msg)
                    print msg

    @ExceptionCatcher("self.run_once_failed")
    def run_once(self, envrs, kojitag, **kwargs):
        update_id = kwargs['name'] or kwargs['id']

        # Get a list of all repos we monitor
        # FIXME - perhaps we should only query for 'active' repos
        reponames = [reponame for reponame in repoinfo.repos()]
        # Get a list of koji-tags For all stable repos (includes the requested kojitag)
        repotags = [repoinfo.getrepo(reponame)['tag'] for reponame in reponames \
            if not reponame.endswith('-testing') or repoinfo.getrepo(reponame)['tag'] == kojitag]
        repotags.sort()
        assert kojitag in repotags, 'Requested unsupported kojitag: %s' % kojitag
        current_tag = repotags.index(kojitag)

        hi_tags = repotags[(current_tag + 1):] # tags higher than current
        low_eq_tags = repotags[:(current_tag + 1)] # tags lower or equal

        for envr in envrs:
            msg = '%s\n%s into %s\n%s' % (40*'=', envr, kojitag, 40*'=')
            print msg
            self.log.append(msg)

            # our testing package
            (name, version, release, epoch, arch) = rpmUtils.miscutils.splitFilename(envr)
            matching_build = {
                    'name': name,
                    'version' : version,
                    'release' : release + '.' + arch,
                    'epoch' : epoch,
                    'envr' : envr,
                    }

            if kojitag.find('updates') == -1 and repotags[current_tag] != repotags[-1]:
                msg = "Warning: Pushing into stable repository"
                self.log.append(msg)
                print msg

            koji = autoqa.koji_utils.SimpleKojiClientSession()
            latest_build = koji.latest_by_tag(kojitag, matching_build['name'])
            if latest_build is not None:
                result = autoqa.koji_utils.compareEVR(matching_build, latest_build)
                if result <= 0:
                    msg = "Warning: Same or newer build already exists: %s" % latest_build['nvr']
                    self.log.append(msg)
                    print msg

            # compare with lower or equal tags, so version has to be greater or equal
            self.compare(matching_build, low_eq_tags, operator.ge, '>=')
            # compare with higher tags, so version has to be lower or equal
            self.compare(matching_build, hi_tags, operator.le, '<=')

        summary = str(len(envrs) - len(self.envr_list)) + ' OK, ' + str(len(self.envr_list)) + ' FAILED'
        msg = 'SUMMARY: ' + summary
        self.log.append(msg)
        print msg

        self.result = self.test_result
        self.outputs = ""
        for i in self.log:
            self.outputs += i + '\n'
        self.outputs += '\n \n'
        self.summary = '%s for %s' % (summary, update_id)

